@@ -1,8 +1,31 @@
 #!/usr/local/bin/python
 '''
-Created on 1 nov. 2009
+:py:mod:`obicount` : Count the number of sequences 
+==================================================
 
-@author: coissac
+.. codeauthor:: Eric Coissac <eric.coissac@metabarcoding.org>
+
+:py:mod:`obicount` count the number of sequences in a sequence file and/or their total count (use the 'count' annotation tag).
+
+Example: To get the number of sequences in a file :
+
+.. code-block:: bash
+
+   > obicount seq.fasta 
+
+
+Example: To get the total count of the sequences in a file:
+
+.. code-block:: bash
+
+   > obicount -a seq.fasta
+   
+Example: To get both the number and the total count of the sequences in a file :
+
+.. code-block:: bash
+
+   > obicount -s seq.fasta
+    
 '''
 
 from obitools.options import getOptionManager
@@ -12,13 +35,13 @@ def addCountOptions(optionManager):
     optionManager.add_option('-s','--sequence',
                              action="store_true", dest="sequence",
                              default=False,
-                             help="Print the count of differentes sequences"
+                             help="Print the number sequences and their total count"
                              )
  
     optionManager.add_option('-a','--all',
                              action="store_true", dest="all",
                              default=False,
-                             help="Print the count of all sequences"
+                             help="Print the total count of the sequences (if a sequence has no 'count' tag its default count is 1)"
                              )
 
 
