@@ -1,12 +1,27 @@
 #!/usr/local/bin/python
-'''
-Created on 6 juil. 2010
+"""
+:py:mod:`obiselect` : Select and print sequences identified by a list of IDs 
+============================================================================
 
-@author: coissac
-'''
+.. codeauthor:: Eric Coissac <eric.coissac@metabarcoding.org>
+
+:py:mod:`obiselect` command allows to select a set of sequences identified by a list of IDs.
+
+Note that the supplied identifiers are not required to correspond to any existing sequence identifier.
+If an ID is not found, no warning will be produced.
+
+Option -v allows to invert the selection and thus select sequences whose IDs are not in the list.
+
+Example:
+
+.. code-block:: bash
+
+   > obiselect --identifier IDs.list seq.fasta > selectedSeq.fasta
+
+
+"""
 from obitools.format.options import addInOutputOption, sequenceWriterGenerator
 from obitools.options import getOptionManager
-from obitools.format.sequence import autoSequenceIterator
 
 def addSelectOptions(optionManager):
     
@@ -27,7 +42,7 @@ def addSelectOptions(optionManager):
 
 
 if __name__ == '__main__':
-    optionParser = getOptionManager([addSelectOptions,addInOutputOption])
+    optionParser = getOptionManager([addSelectOptions,addInOutputOption], progdoc=__doc__)
     
     (options, entries) = optionParser()
     
