@@ -1,21 +1,22 @@
 '''
-obitools main module.
+**obitools** main module
+------------------------
+
+.. codeauthor:: Eric Coissac <eric.coissac@metabarcoding.org>
 
-@author: Eric Coissac
-@contact: eric@coissac.eu
-@license: CeCILL 2.0
 
 
 obitools module provides base class for sequence manipulation.
 
-All biological sequences must be subclass of L{obitools.BioSequence}.
+All biological sequences must be subclass of :py:class:`obitools.BioSequence`.
 Some biological sequences are defined as transformation of other
 biological sequences. For example Reversed complemented sequences
-are a transformation of a L{obitools.NucSequence}. This particular
-type of sequences are subclasses of the L{obitools.WrappedBioSequence}.
+are a transformation of a :py:class:`obitools.NucSequence`. This particular
+type of sequences are subclasses of the :py:class:`obitools.WrappedBioSequence`.
 
+.. inheritance-diagram:: BioSequence NucSequence AASequence WrappedBioSequence SubSequence DNAComplementSequence
+        :parts: 1
 
-G{packagetree obitools}
 
 '''
 
@@ -23,6 +24,9 @@ from weakref import ref
 
 from obitools.utils.iterator import uniqueChain
 from itertools import chain
+import re
+
+_default_raw_parser = " %s *= *([^;]*);"
 
 try:
     from functools import partial
@@ -69,28 +73,33 @@ class BioSequence(object):
         - a definition an manage 
         - a set of complementary information on a key / value principle.
     
-    BioSequence is an abstract class and must be instanciated
-    from its subclasses
+    .. warning:: 
+        
+            :py:class:`obitools.BioSequence` is an abstract class, this constructor
+            can only be called by a subclass constructor.
     '''
     
-    def __init__(self,id,seq,definition=None,evaluated=True,**info):
-        '''
-        BioSequence constructor.
+    def __init__(self,id,seq,definition=None,rawinfo=None,rawparser=_default_raw_parser,**info):
+        '''        
         
-        C{obitools.BioSequence} is an abstract class, this constructor
-        can only be called by a subclass constructor.
-        
-        @param id: sequence identifier
-        @type id:  str
+        :param id: sequence identifier
+        :type id:  `str`
  
-        @param seq: the sequence
-        @type seq:  str
+        :param seq: the sequence
+        :type seq:  `str`
 
-        @param definition: sequence definition (optional)
-        @type definition: str
-
-        @param info: extra named parameters can be added to associate complementary
-        data to the sequence
+        :param definition: sequence definition (optional)
+        :type definition: `str`
+        
+        :param rawinfo: a text containing a set of key=value; patterns
+        :type definition: `str`
+        
+        :param rawparser: a text describing a regular patterns template 
+                          used to parse rawinfo
+        :type definition: `str`
+        
+        :param info: extra named parameters can be added to associate complementary
+                     data to the sequence
         
         '''
         
@@ -98,12 +107,25 @@ class BioSequence(object):
         
         self._seq=str(seq).lower()
         self._info = dict(info)
-        self._eval = {}
-        for k in self._info:
-            self._eval[k]=evaluated
+        if rawinfo is not None:
+            self._rawinfo=' ' + rawinfo
+        else:
+            self._rawinfo=None
+        self._rawparser=rawparser
         self.definition=definition
         self.id=id
         self._hasTaxid=None
+
+    def get_seq(self):
+        return self.__seq
+
+
+    def set_seq(self, value):
+        if not isinstance(value, str):
+            value=str(value)
+        self.__seq = value
+        self.__len = len(value)
+
         
     def clone(self):
         seq = type(self)(self.id,
@@ -111,7 +133,8 @@ class BioSequence(object):
                          definition=self.definition
                          )
         seq._info=dict(self.getTags())
-        seq._eval=dict(self._eval)
+        seq._rawinfo=self._rawinfo
+        seq._rawparser=self._rawparser
         seq._hasTaxid=self._hasTaxid
         return seq
         
@@ -119,8 +142,8 @@ class BioSequence(object):
         '''
         Sequence definition getter.
         
-            @return: the sequence definition
-            @rtype: str
+        :return: the sequence definition
+        :rtype: str
             
         '''
         return self._definition
@@ -129,9 +152,9 @@ class BioSequence(object):
         '''
         Sequence definition setter.
         
-            @param value: the new sequence definition
-            @type value: C{str}
-            @return: C{None}
+        :param value: the new sequence definition
+        :type value: C{str}
+        :return: C{None}
         '''
         self._definition = value
 
@@ -139,8 +162,8 @@ class BioSequence(object):
         '''
         Sequence identifier getter
         
-            @return: the sequence identifier
-            @rtype: C{str}
+        :return: the sequence identifier
+        :rtype: C{str}
         '''
         return self._id
 
@@ -148,9 +171,9 @@ class BioSequence(object):
         '''
         Sequence identifier setter.
         
-        @param value: the new sequence identifier
-        @type value:  C{str}
-        @return: C{None}
+        :param value: the new sequence identifier
+        :type value:  C{str}
+        :return: C{None}
         '''
         self._id = value
 
@@ -158,8 +181,8 @@ class BioSequence(object):
         '''
         Return the sequence as a string
         
-            @return: the string representation of the sequence
-            @rtype: str
+        :return: the string representation of the sequence
+        :rtype: str
         '''
         return self._seq
     
@@ -167,13 +190,13 @@ class BioSequence(object):
         '''
         Return the symbole at C{position} in the sequence
         
-        @param position: the desired position. Position start from 0
+        :param position: the desired position. Position start from 0
                          if position is < 0 then they are considered
                          to reference the end of the sequence.
-        @type position: C{int}
+        :type position: `int`
         
-        @return: a one letter string
-        @rtype: C{str}
+        :return: a one letter string
+        :rtype: `str`
         '''
         return str(self)[position]
     
@@ -185,11 +208,12 @@ class BioSequence(object):
         an interger or a python C{slice} instance. If C{location}
         is an iterger this method is equivalent to L{getSymbolAt}.
         
-        @param location: the positions of the subsequence to return
-        @type location: C{Location} or C{int} or C{slice}
-        @return: the subsequence
-        @rtype: a single character as a C{str} is C{location} is an integer,
+        :param location: the positions of the subsequence to return
+        :type location: C{Location} or C{int} or C{slice}
+        :return: the subsequence
+        :rtype: a single character as a C{str} is C{location} is an integer,
                 a L{obitools.SubSequence} instance otherwise.
+        
         '''
         if isinstance(location,Location):
             return location.extractSequence(self)
@@ -201,13 +225,30 @@ class BioSequence(object):
         raise TypeError,'key must be a Location, an integer or a slice'  
     
     def getKey(self,key):
-        if not self._eval[key]:
-            self._eval[key]=True
-            try:
-                self._info[key]=eval(self._info[key])
-            except:
-                pass
-        return self._info[key]
+        if key not in self._info:
+            if self._rawinfo is None:
+                if key=='count':
+                    return 1
+                else:
+                    raise KeyError,key
+            p = re.compile(self._rawparser % key)
+            m = p.search(self._rawinfo)
+            if m is not None:
+                v=m.group(1)
+                self._rawinfo=' ' + self._rawinfo[0:m.start(0)]+self._rawinfo[m.end(0):]
+                try:
+                    v = eval(v)
+                except:
+                    pass
+                self._info[key]=v
+            else:
+                if key=='count':
+                    v=1
+                else:
+                    raise KeyError,key
+        else:
+            v=self._info[key]
+        return v
             
     def extractTaxon(self):
         '''
@@ -215,7 +256,7 @@ class BioSequence(object):
         This method by default return None. It should be subclassed
         if necessary as in L{obitools.seqdb.AnnotatedSequence}.
         
-            @return: None
+        :return: None
         '''
         self._hasTaxid=self.hasKey('taxid')
         return None
@@ -232,17 +273,20 @@ class BioSequence(object):
             return self.getSubSeq(key)
         
     def __setitem__(self,key,value):
+        self.__contains__(key)
         self._info[key]=value
-        self._eval[key]=True
         if key=='taxid':
             self._hasTaxid=value is not None
         
     def __delitem__(self,key):
         if isinstance(key, str):
-            del self._info[key]
-            del self._eval[key]
+            if key in self:
+                del self._info[key]
+            else:
+                raise KeyError,key    
+            
             if key=='taxid':
-                self._hasTaxid=None
+                self._hasTaxid=False
         else:
             raise TypeError,key
         
@@ -253,10 +297,25 @@ class BioSequence(object):
         return iter(str(self))
     
     def __len__(self):
-        return len(str(self))
+        return self.__len
     
     def hasKey(self,key):
-        return key in self._info
+        rep = key in self._info
+        
+        if not rep and self._rawinfo is not None:
+            p = re.compile(self._rawparser % key)
+            m = p.search(self._rawinfo)
+            if m is not None:
+                v=m.group(1)
+                self._rawinfo=' ' + self._rawinfo[0:m.start(0)]+self._rawinfo[m.end(0):]
+                try:
+                    v = eval(v)
+                except:
+                    pass
+                self._info[key]=v
+                rep=True
+        
+        return rep
     
     def __contains__(self,key):
         '''
@@ -265,10 +324,10 @@ class BioSequence(object):
         The C{in} operator test if the C{key} value is defined for this
         sequence.
          
-        @param key: the name of the checked value
-        @type key: str
-        @return: C{True} if the value is defined, {False} otherwise.
-        @rtype: C{bool}
+        :param key: the name of the checked value
+        :type key: str
+        :return: C{True} if the value is defined, {False} otherwise.
+        :rtype: C{bool}
         '''
         if key=='taxid' and self._hasTaxid is None:
             self.extractTaxon()
@@ -283,29 +342,32 @@ class BioSequence(object):
         associated to the sequence. It works similarly to
         the iteritems function of C{dict}.
         
-            @return: an iterator over the items (key,value)
-                     link to a sequence
-            @rtype: iterator over tuple
-            @see: L{items}
+        :return: an iterator over the items (key,value)
+                 link to a sequence
+        :rtype: iterator over tuple
+        :see: L{items}
         '''
-        for k in self._info:
-            try:
-                self._eval[k]=True
-                self._info[k]=eval(self._info[k])
-            except:
-                pass
+        if self._rawinfo is not None:
+            p = re.compile(self._rawparser % "([a-zA-Z]\w*)")
+            for k,v in p.findall(self._rawinfo):
+                try:
+                    self._info[k]=eval(v)
+                except:
+                    self._info[k]=v
+            self._rawinfo=None
         return self._info.iteritems()
     
     def items(self):
         return [x for x in self.iteritems()]
     
     def iterkeys(self):
-        return self._info.iterkeys()
+        return (k for k,v in self.iteritems())
     
     def keys(self):
         return [x for x in self.iterkeys()]
     
     def getTags(self):
+        self.iteritems()
         return self._info
     
     def getRoot(self):
@@ -327,7 +389,7 @@ class BioSequence(object):
     definition = property(getDefinition, setDefinition, None, "Sequence Definition")
 
     id = property(getId, setId, None, 'Sequence identifier')
-    
+        
     def _getTaxid(self):
         return self['taxid']
     
@@ -335,10 +397,21 @@ class BioSequence(object):
         self['taxid']=taxid
         
     taxid = property(_getTaxid,_setTaxid,None,'NCBI Taxonomy identifier')
+    _seq = property(get_seq, set_seq, None, None)
         
 class NucSequence(BioSequence):
+    """
+    :py:class:`NucSequence` specialize the :py:class:`BioSequence` class for storing DNA
+    sequences. 
+    
+    The constructor is identical to the :py:class:`BioSequence` constructor.
+    """
  
     def complement(self):
+        """
+        :return: The reverse complemented sequence as an instance of :py:class:`DNAComplementSequence`
+        :rtype: :py:class:`DNAComplementSequence`
+        """
         return DNAComplementSequence(self)
     
     def isNucleotide(self):
@@ -346,30 +419,44 @@ class NucSequence(BioSequence):
     
 
 class AASequence(BioSequence):
+    """
+    :py:class:`AASequence` specialize the :py:class:`BioSequence` class for storing protein
+    sequences. 
+    
+    The constructor is identical to the :py:class:`BioSequence` constructor.
+    """
+ 
 
     def isNucleotide(self):
         return False
     
    
 class WrappedBioSequence(BioSequence):
+    """
+    .. warning:: 
+        
+            :py:class:`obitools.WrappedBioSequence` is an abstract class, this constructor
+            can only be called by a subclass constructor.
+    """
+ 
+
+    def __init__(self,reference,id=None,definition=None,**info):
+
+        assert type(self)!=WrappedBioSequence,"obitools.WrappedBioSequence is an abstract class"
 
-    def __init__(self,reference,id=None,definition=None,evaluated=True,**info):
         self._wrapped = reference
         reference.register(self)
         self._id=id
         self.definition=definition
         self._info=info
-        self._eval = {}
-        for k in self._info:
-            self._eval[k]=evaluated
-            
+        
     def clone(self):
         seq = type(self)(self.wrapped,
                          id=self._id,
                          definition=self._definition
                          )
         seq._info=dict(self._info)
-        seq._eval=dict(self._eval)
+        
         return seq
         
     def getWrapped(self):
@@ -402,12 +489,6 @@ class WrappedBioSequence(BioSequence):
             
     def getKey(self,key):
         if key in self._info:
-            if not self._eval[key]:
-                try:
-                    self._eval[key]=True
-                    self._info[key]=eval(self._info[key])
-                except:
-                    pass
             return self._info[key]
         else:
             return self.wrapped.getKey(key)
@@ -431,17 +512,37 @@ class WrappedBioSequence(BioSequence):
     def getRoot(self):
         return self.wrapped.getRoot()
     
+    def complement(self):
+        """
+        The :py:meth:`complement` method of the :py:class:`WrappedBioSequence` class 
+        raises an exception :py:exc:`AttributeError` if the method is called and the cut
+        sequence does not corresponds to a nucleic acid sequence.
+        """
+        
+        if self.wrapped.isNucleotide():
+            return DNAComplementSequence(self)
+        raise AttributeError
+
+    
     def _posInWrapped(self,position):
         return position
     
     
-    definition = property(getDefinition,BioSequence.setDefinition, None, "Sequence Definition")
-    id = property(getId,BioSequence.setId, None, "Sequence Identifier")
+    definition = property(getDefinition,BioSequence.setDefinition, None)
+    id = property(getId,BioSequence.setId, None)
 
-    wrapped = property(getWrapped, None, None, "Wrapped's Docstring")
-         
+    wrapped = property(getWrapped, None, None, "A pointer to the wrapped sequence")
+    
+    def  _getWrappedRawInfo(self):
+        return self.wrapped._rawinfo
+    
+    _rawinfo = property(_getWrappedRawInfo)
+        
     
 class SubSequence(WrappedBioSequence):
+    """
+    """
+ 
     
     @staticmethod
     def _sign(x):
@@ -455,8 +556,8 @@ class SubSequence(WrappedBioSequence):
                  location=None,
                  start=None,stop=None,
                  id=None,definition=None,
-                 evaluated=True,**info):
-        WrappedBioSequence.__init__(self,reference,id=None,definition=None,evaluated=evaluated,**info)
+                 **info):
+        WrappedBioSequence.__init__(self,reference,id=None,definition=None,**info)
         
         if isinstance(location, slice):
             self._location = location
@@ -472,7 +573,6 @@ class SubSequence(WrappedBioSequence):
         self._xrange=xrange(*self._indices)
  
         self._info['cut']='[%d,%d,%s]' % self._indices
-        self._eval['cut']=True
         
         if hasattr(reference,'quality'):
             self.quality = reference.quality[self._location]
@@ -502,16 +602,17 @@ class SubSequence(WrappedBioSequence):
     def _posInWrapped(self,position):
         return self._xrange[position]
     
-    def complement(self):
-        if self.wrapped.isNucleotide():
-            return DNAComplementSequence(self)
-        raise AttributeError
     
-    id = property(getId,BioSequence.setId, None, "Sequence Identifier")
+    id = property(getId,BioSequence.setId, None)
 
                 
  
 class DNAComplementSequence(WrappedBioSequence):
+    """
+    Class used to represent a reverse complemented DNA sequence. Usually instances
+    of this class are produced by using the :py:meth:`NucSequence.complement` method.
+    """
+ 
 
     _comp={'a': 't', 'c': 'g', 'g': 'c', 't': 'a',
            'r': 'y', 'y': 'r', 'k': 'm', 'm': 'k', 
@@ -521,11 +622,10 @@ class DNAComplementSequence(WrappedBioSequence):
 
     
     def __init__(self,reference,
-                 id=None,definition=None,evaluated=True,**info):
-        WrappedBioSequence.__init__(self,reference,id=None,definition=None,evaluated=evaluated,**info)
+                 id=None,definition=None,**info):
+        WrappedBioSequence.__init__(self,reference,id=None,definition=None,**info)
         assert reference.isNucleotide()
         self._info['complemented']=True
-        self._eval['complemented']=True
         if hasattr(reference,'quality'):
             self.quality = reference.quality[::-1]
 
@@ -550,9 +650,14 @@ class DNAComplementSequence(WrappedBioSequence):
         return DNAComplementSequence._comp[self.wrapped.getSymbolAt(self.posInWrapped(position))]
     
     def complement(self):
+        """
+        The :py:meth:`complement` method of the :py:class:`DNAComplementSequence` class actually
+        returns the wrapped sequenced. Effectively the reversed complemented sequence of a reversed
+        complemented sequence is the initial sequence.
+        """
         return self.wrapped
     
-    id = property(getId,BioSequence.setId, None, "Sequence Identifier")
+    id = property(getId,BioSequence.setId, None)
                 
    
 def _isNucSeq(text):
@@ -566,10 +671,42 @@ def _isNucSeq(text):
             notnuc+=1
     return notnuc==0 and float(acgt) > ltot
 
+ 
+def bioSeqGenerator(id,seq,definition=None,rawinfo=None,rawparser=_default_raw_parser,**info):
+    """
+    Generate automagically the good class instance between :
+    
+        - :py:class:`NucSequence`
+        - :py:class:`AASequence`
+    
+    Build a new sequence instance. Sequences are instancied as :py:class:`NucSequence` if the
+    `seq` attribute contains more than 80% of *A*, *C*, *G*, *T* or *-* symbols 
+    in upper or lower cases. Conversely, the new sequence instance is instancied as 
+    :py:class:`AASequence`.
+    
 
-def bioSeqGenerator(id,seq,definition=None,evaluated=True,**info):
+    
+    :param id: sequence identifier
+    :type id:  `str`
+    
+    :param seq: the sequence
+    :type seq:  `str`
+    
+    :param definition: sequence definition (optional)
+    :type definition: `str`
+    
+    :param rawinfo: a text containing a set of key=value; patterns
+    :type definition: `str`
+    
+    :param rawparser: a text describing a regular patterns template 
+                      used to parse rawinfo
+    :type definition: `str`
+    
+    :param info: extra named parameters can be added to associate complementary
+                 data to the sequence
+    """
     if _isNucSeq(seq):
-        return NucSequence(id,seq,definition,evaluated,**info)
+        return NucSequence(id,seq,definition,rawinfo,rawparser,**info)
     else:
-        return AASequence(id,seq,definition,evaluated,**info)
+        return AASequence(id,seq,definition,rawinfo,rawparser,**info)
     
