@@ -34,6 +34,7 @@ column_t* allocateColumn(int length,column_t *column, bool mode8bits)
 
 		column->size = 0;
 		column->data.shrt=NULL;
+		column->score.shrt=NULL;
 		newc = true;
 	}
 
@@ -43,10 +44,16 @@ column_t* allocateColumn(int length,column_t *column, bool mode8bits)
 	if (size > column->size)
 	{
 		int16_t *old = column->data.shrt;
+		int16_t *olds= column->score.shrt;
+
 		column->data.shrt = malloc(size);
-		if (!column->data.shrt)
+		column->score.shrt= malloc(size);
+
+		if (!column->data.shrt || !column->score.shrt)
 		{
 			column->data.shrt = old;
+			column->score.shrt= olds;
+
 			if (newc)
 			{
 				free(column);
@@ -68,12 +75,16 @@ void freeColumn(column_p column)
 	{
 		if (column->data.shrt)
 			free(column->data.shrt);
+
+		if (column->score.shrt)
+			free(column->score.shrt);
+
 		free(column);
 	}
 }
 
-int fastLCSScore(const char* seq1, const char* seq2)
+int fastLCSScore(const char* seq1, const char* seq2,column_pp column,int32_t* lpath)
 {
-	return fastLCSScore16(seq1,seq2,NULL);
+	return fastLCSScore16(seq1,seq2,column,lpath);
 }
 
