@@ -11,12 +11,18 @@ typedef struct {
 	        int8_t  *byte;
 	      } data;
 
+	union { int16_t *shrt;
+			int8_t  *byte;
+		  } score;
+
+
 } column_t, **column_pp, *column_p;
 
 column_p allocateColumn(int length,column_t *column, bool mode8bits);
+
 void freeColumn(column_p column);
 
-int fastLCSScore16(const char* seq1, const char* seq2,column_pp ppcolumn);
-int fastLCSScore8(const char* seq1, const char* seq2,column_pp ppcolumn);
+int fastLCSScore16(const char* seq1, const char* seq2,column_pp ppcolumn,int* lpath);
+int fastLCSScore8(const char* seq1, const char* seq2,column_pp ppcolumn,int* lpath);
 
-int fastLCSScore(const char* seq1, const char* seq2);
+int fastLCSScore(const char* seq1, const char* seq2,column_pp column,int* lpath);
