@@ -204,7 +204,11 @@ int ispossible(int len1, unsigned char *t1, int over1,
 	}
 	else
 	{
-		reflen = MIN(len1,len2);
+		if (large)
+			reflen = MAX(len1,len2);
+		else
+			reflen = MIN(len1,len2);
+
 		lcs = (int32_t) minimum;
 	}
 
@@ -213,7 +217,7 @@ int ispossible(int len1, unsigned char *t1, int over1,
 
 	mincount = thresholdLCS4(reflen,lcs);
 
-	// fprintf(stderr,"MaxLCS %d : %d\n",compareTable(seq1->table,seq1->over,seq2->table,seq2->over),mincount);
+// fprintf(stderr,"MaxLCS %d %d %d : %d\n",reflen,lcs,compareTable(t1,over1,t2,over2),mincount);
 
 	return compareTable(t1,over1,t2,over2) >=mincount;
 }
