@@ -97,15 +97,19 @@ class EcoPCRDBSequenceWriter(object):
         else:
             taxon=self._taxonomy.findIndex(seq['taxid'])
         
-        packed = struct.pack('> I i 20s I I I %ds %ds' % (delength,cptseqlength),
-                             totalSize,
-                             taxon,
-                             seq.id,
-                             delength,
-                             len(seq),
-                             cptseqlength,
-                             seq.definition,
-                             compactseq)
+        try:
+            packed = struct.pack('> I i 20s I I I %ds %ds' % (delength,cptseqlength),
+                                 totalSize,
+                                 taxon,
+                                 seq.id,
+                                 delength,
+                                 len(seq),
+                                 cptseqlength,
+                                 seq.definition,
+                                 compactseq)
+        except struct.error as e:
+            print >>sys.stderr,"\n\n============\n\nError on sequence : %s\n\n" % seq.id
+            raise e
         
         assert len(packed) == totalSize+4, "error in sequence packing"
     
