@@ -15,35 +15,46 @@ class Taxonomy(object):
         @param path: path to the ecoPCR database including the database prefix name
         @type path: C{str}
         '''
+        
+        self._ranks.append('obi')
+
         self._speciesidx = self._ranks.index('species')
         self._genusidx   = self._ranks.index('genus')
         self._familyidx   = self._ranks.index('family')
         self._orderidx   = self._ranks.index('order')
         self._nameidx=dict((x[0],x[2]) for x in self._name)
-
-    def findTaxonByTaxid(self,taxid):
-        if taxid is None:
-            return None
-        taxid = self._index[taxid]
+        self._nameidx.update(dict((x[0],x[2]) for x in self._preferedName))
+        self._preferedidx=dict((x[2],x[1]) for x in self._preferedName)
         
+        self._bigestTaxid = max(x[0] for x in self._taxonomy)
+        
+
+    def findTaxonByIdx(self,idx):
+        if idx is None:
+            return None
+        return self._taxonomy[idx]
+
+    def findIndex(self,taxid):
         if taxid is None:
             return None
-
-        return self._taxonomy[taxid]
-    
+        return self._index[taxid]
+        
+    def findTaxonByTaxid(self,taxid):
+        return self.findTaxonByIdx(self.findIndex(taxid))
+        
     def findTaxonByName(self,name):
         return self._taxonomy[self._nameidx[name]]
     
-    
-    
     def findRankByName(self,rank):
         try:
             return self._ranks.index(rank)
         except ValueError:
             return None
         
-    def findIndex(self,taxid):
-        return self._index[taxid]
+    def __contains__(self,taxid):
+        return self.findTaxonByTaxid(taxid) is not None
+    
+        
 
 
     #####
@@ -55,7 +66,7 @@ class Taxonomy(object):
 
     def subTreeIterator(self, taxid):
         "return subtree for given taxonomic id "
-        idx = self._index[taxid]
+        idx = self.findTaxonByTaxid(taxid)
         yield self._taxonomy[idx]
         for t in self._taxonomy:
             if t[2] == idx:
@@ -109,8 +120,13 @@ class Taxonomy(object):
         lca = self.lastCommonTaxon(*taxids)
         idx = self._index[lca]
         sublca = [t[0] for t in self._taxonomy if t[2]==idx]
-        dist={}
         return sublca
+
+    
+    def getPreferedName(self,taxid):
+        idx = self.findIndex(taxid)
+        return self._preferedidx.get(idx,self._taxonomy[idx][3])
+    
     
     def getScientificName(self,taxid):
         return self.findTaxonByTaxid(taxid)[3]
@@ -152,12 +168,50 @@ class Taxonomy(object):
         assert len(a)==1,"All taxa must have the same parent"
         newtaxid=max([2999999]+[x[0] for x in self._taxonomy if x[0]>=3000000 and x[0]<4000000])+1
         newidx=len(self._taxonomy)
-        if 'MOTU' not in self._ranks:
-            self._ranks.append('MOTU')
-        rankid=self._ranks.index('MOTU')
+        if 'GROUP' not in self._ranks:
+            self._ranks.append('GROUP')
+        rankid=self._ranks.index('GROUP')
         self._taxonomy.append((newtaxid,rankid,a.pop(),groupname))
         for x in t:
             x[2]=newidx
+            
+    def addLocalTaxon(self,name,rank,parent,minimaltaxid=10000000):
+        newtaxid = minimaltaxid if (self._bigestTaxid < minimaltaxid) else self._bigestTaxid+1
+        
+        rankid=self.findRankByName(rank)
+        parentidx = self.findIndex(int(parent))
+        tx = (newtaxid,rankid,parentidx,name,'local')
+        self._taxonomy.append(tx)
+        newidx=len(self._taxonomy)-1
+        self._name.append((name,'scientific name',newidx))
+        self._nameidx[name]=newidx
+        self._index[newtaxid]=newidx
+
+        self._bigestTaxid=newtaxid
+        
+        return newtaxid
+    
+    def removeLocalTaxon(self,taxid):
+        raise NotImplemented
+        txidx = self.findIndex(taxid)
+        taxon = self.findTaxonByIdx(txidx)
+
+        assert txidx >= self._localtaxon,"Only local taxon can be deleted"
+
+        for t in self._taxonomy:
+            if t[2] == txidx:
+                self.removeLocalTaxon(t[0])
+                
+        
+        
+        
+        return taxon
+    
+    def addPreferedName(self,taxid,name):
+        idx = self.findIndex(taxid)
+        self._preferedName.append(name,'obi',idx)
+        self._preferedidx[idx]=name
+        return taxid
 
 class EcoTaxonomyDB(Taxonomy,EcoPCRDBFile):
     '''
@@ -174,8 +228,10 @@ class EcoTaxonomyDB(Taxonomy,EcoPCRDBFile):
         '''
         self._path = path
         self._taxonFile =  "%s.tdx" % self._path
+        self._localTaxonFile =  "%s.ldx" % self._path
         self._ranksFile =  "%s.rdx" % self._path
         self._namesFile =  "%s.ndx" % self._path
+        self._preferedNamesFile =  "%s.pdx" % self._path
         self._aliasFile =  "%s.adx" % self._path
         
         print >> sys.stderr,"Reading binary taxonomy database...",
@@ -193,8 +249,8 @@ class EcoTaxonomyDB(Taxonomy,EcoPCRDBFile):
     #
     #####
                    
-    def __ecoNameIterator(self):
-        for record in self._ecoRecordIterator(self._namesFile):
+    def __ecoNameIterator(self,file):
+        for record in self._ecoRecordIterator(file):
             lrecord = len(record)
             lnames  = lrecord - 16
             (isScientificName,namelength,classLength,indextaxid,names)=struct.unpack('> I I I I %ds' % lnames, record)
@@ -208,15 +264,27 @@ class EcoTaxonomyDB(Taxonomy,EcoPCRDBFile):
             lrecord = len(record)
             lnames  = lrecord - 16
             (taxid,rankid,parentidx,nameLength,name)=struct.unpack('> I I I I %ds' % lnames, record)
-            yield  (taxid,rankid,parentidx,name)
-                
+            yield  (taxid,rankid,parentidx,name,'ncbi')
+            
+        try :
+            lt=0
+            for record in self._ecoRecordIterator(self._localTaxonFile):
+                lrecord = len(record)
+                lnames  = lrecord - 16
+                (taxid,rankid,parentidx,nameLength,name)=struct.unpack('> I I I I %ds' % lnames, record)
+                lt+=1
+                yield  (taxid,rankid,parentidx,name,'local')
+            print >> sys.stderr, " [INFO : Local taxon file found] : %d added taxa" % lt
+        except:
+            print >> sys.stderr, " [INFO : Local taxon file not found] "
+            
     def __ecoRankIterator(self):
         for record in self._ecoRecordIterator(self._ranksFile):
             yield  record
     
     def __ecoAliasIterator(self):
         for record in self._ecoRecordIterator(self._aliasFile):
-            (taxid,index) = struct.unpack('> I I',record)
+            (taxid,index) = struct.unpack('> I i',record)
             yield taxid,index
             
     #####
@@ -226,7 +294,7 @@ class EcoTaxonomyDB(Taxonomy,EcoPCRDBFile):
     #####
     
     def __ecoNameIndex(self):
-        indexName = [x for x in self.__ecoNameIterator()]
+        indexName = [x for x in self.__ecoNameIterator(self._namesFile)]
         return indexName
 
     def __ecoRankIndex(self):
@@ -238,26 +306,49 @@ class EcoTaxonomyDB(Taxonomy,EcoPCRDBFile):
         
         try :
             index = dict(self.__ecoAliasIterator())
-            print >> sys.stderr, " [Alias file found] ",
+            print >> sys.stderr, " [INFO : Taxon alias file found] "
             buildIndex=False
         except:
-            print >> sys.stderr, " [Alias file not found] ",
+            print >> sys.stderr, " [INFO : Taxon alias file not found] "
             index={}
             i = 0;
             buildIndex=True
             
-            
+        localtaxon=0
+        i=0
         for x in self.__ecoTaxonomicIterator():
             taxonomy.append(x)
-            if buildIndex:
+            if x[4]=='ncbi':
+                localtaxon+=1
+                
+            if buildIndex or x[4]!='ncbi':
                 index[x[0]] = i 
-                i+=1
-        return taxonomy, index
+            i+=1
+            
+                
+        print  >> sys.stderr,"Taxonomical tree read",
+        return taxonomy, index,localtaxon
 
     def __readNodeTable(self):
-        self._taxonomy, self._index = self.__ecoTaxonomyIndex()
+        self._taxonomy, self._index, self._localtaxon= self.__ecoTaxonomyIndex()
         self._ranks = self.__ecoRankIndex()
-        self._name = self.__ecoNameIndex()
+        self._name  = self.__ecoNameIndex()
+        
+        # Add local taxon tame to the name index
+        i=self._localtaxon
+        for t in self._taxonomy[self._localtaxon:]:
+            self._name.append((t[3],'scientific name',i))
+            i+=1
+                
+        try :
+            self._preferedName = [(x[0],'obi',x[2]) 
+                                  for x in self.__ecoNameIterator(self._preferedNamesFile)]
+            print >> sys.stderr, " [INFO : Prefered taxon name file found] : %d added taxa" % len(self._preferedName)
+        except:
+            print >> sys.stderr, " [INFO : Prefered taxon name file not found]"
+            self._preferedName = []
+            
+                
     
 
 class TaxonomyDump(Taxonomy):  
@@ -273,7 +364,7 @@ class TaxonomyDump(Taxonomy):
         for taxid,name,classname in self._nameIterator('%s/names.dmp' % taxdir):
             self._name.append((name,classname,self._index[taxid]))
             if classname == 'scientific name':
-                self._taxonomy[self._index[taxid]].append(name)
+                self._taxonomy[self._index[taxid]].extend([name,'ncbi'])
             
         print >>sys.stderr,"Adding taxid alias..."
         for taxid,current in self._mergedNodeIterator('%s/merged.dmp' % taxdir):
@@ -283,6 +374,9 @@ class TaxonomyDump(Taxonomy):
         for taxid in self._deletedNodeIterator('%s/delnodes.dmp' % taxdir):
             self._index[taxid]=None
             
+        self._nameidx=dict((x[0],x[2]) for x in self._name)
+
+            
     def _taxonCmp(t1,t2):
         if t1[0] < t2[0]:
             return -1
@@ -336,6 +430,7 @@ class TaxonomyDump(Taxonomy):
         taxonomy.sort(TaxonomyDump._taxonCmp)
 
         self._taxonomy=taxonomy
+        self._localtaxon=len(taxonomy)
     
         print >>sys.stderr,"Indexing taxonomy..."
         index = {}
@@ -349,6 +444,7 @@ class TaxonomyDump(Taxonomy):
          
         self._ranks=ranks
         self._index=index 
+        self._preferedName = []
 
     def _nameIterator(self,file):
         file = universalOpen(file)
@@ -383,7 +479,7 @@ class TaxonomyDump(Taxonomy):
 #
 #####
 
-def ecoTaxonomyWriter(prefix, taxonomy):
+def ecoTaxonomyWriter(prefix, taxonomy,onlyLocal=False):
 
     def ecoTaxPacker(tx):
         
@@ -446,12 +542,31 @@ def ecoTaxonomyWriter(prefix, taxonomy):
     
     def ecoTaxWriter(file,taxonomy):
         output = open(file,'wb')
-        output.write(struct.pack('> I',len(taxonomy)))
-        
+        nbtaxon = reduce(lambda x,y:x+y,(1 for t in taxonomy if t[4]=='ncbi'),0)
+
+        output.write(struct.pack('> I',nbtaxon))
+                     
         for tx in taxonomy:
-            output.write(ecoTaxPacker(tx))
-    
+            if tx[4]=='ncbi':
+                output.write(ecoTaxPacker(tx))
+                
         output.close()
+        return nbtaxon < len(taxonomy)
+
+    def ecoLocalTaxWriter(file,taxonomy):
+        nbtaxon = reduce(lambda x,y:x+y,(1 for t in taxonomy if t[4]!='ncbi'),0)
+
+        if nbtaxon:
+            output = open(file,'wb')
+    
+            output.write(struct.pack('> I',nbtaxon))
+                         
+            for tx in taxonomy:
+                if tx[4]!='ncbi':
+                    output.write(ecoTaxPacker(tx))
+                    
+            output.close()
+
         
     def ecoRankWriter(file,ranks):
         output = open(file,'wb')
@@ -495,8 +610,21 @@ def ecoTaxonomyWriter(prefix, taxonomy):
     
         output.close()
         
+    def ecoPreferedNameWriter(file,names):
+        output = open(file,'wb')
+        output.write(struct.pack('> I',len(names)))
+        for name in names:
+            output.write(ecoNamePacker(name))
     
-    ecoRankWriter('%s.rdx' % prefix, taxonomy._ranks)
-    ecoTaxWriter('%s.tdx' % prefix, taxonomy._taxonomy)
-    ecoNameWriter('%s.ndx' % prefix, taxonomy._name)
-    ecoAliasWriter('%s.adx' % prefix, taxonomy._index)
+        output.close()
+       
+    localtaxon=True
+    if not onlyLocal:
+        ecoRankWriter('%s.rdx' % prefix, taxonomy._ranks)
+        localtaxon = ecoTaxWriter('%s.tdx' % prefix, taxonomy._taxonomy)
+        ecoNameWriter('%s.ndx' % prefix, [x for x in taxonomy._name if x[2] < taxonomy._localtaxon])
+        ecoAliasWriter('%s.adx' % prefix, taxonomy._index)
+    if localtaxon:
+        ecoLocalTaxWriter('%s.ldx' % prefix, taxonomy._taxonomy)
+    if taxonomy._preferedName:
+        ecoNameWriter('%s.pdx' % prefix, taxonomy._preferedName)
