@@ -1,5 +1,16 @@
 '''
-This module offert classes to manipulate graphs
+**obitool.graph** for representing graph structure in obitools
+--------------------------------------------------------------
+
+.. codeauthor:: Eric Coissac <eric.coissac@metabarcoding.org>
+
+
+This module offert classes to manipulate graphs, mainly trough the
+:py:class:`obitools.graph.Graph` class.
+
+.. inheritance-diagram:: Graph DiGraph UndirectedGraph
+    :parts: 2
+
 '''
 
 import sys
@@ -15,9 +26,7 @@ class Indexer(dict):
     '''
     
     def __init__(self):
-        '''
-        Constructor
-        '''
+
         self.__max=0
         self.__reverse=[]
         
@@ -25,15 +34,43 @@ class Indexer(dict):
         '''
         Return the python value associated to an integer index.
         
-        @param index: an index value
-        @type index: integer
+        :param index: an index value
+        :type index: int
         
-        @raise IndexError: if the index is not used in this 
+        :raises: IndexError if the index is not used in this 
                            Indexer instance
         '''
         return self.__reverse[index]
     
     def getIndex(self,key,strict=False):
+        '''
+        Return the index associated to a **key** in the indexer. Two
+        modes are available :
+        
+            - strict mode  :
+                
+                if the key is not known by the :py:class:`Indexer` instance
+                a :py:exc:`KeyError` exception is raised.
+                
+            - non strict mode :
+            
+                in this mode if the requested *key** is absent, it is added to
+                the :py:class:`Indexer` instance and the new index is returned
+                                
+        :param key: the requested key
+        :type key: a hashable python value
+        
+        :param strict: select the looking for mode
+        :type strict: bool
+        
+        :return: the index corresponding to the key
+        :rtype: int
+        
+        :raises: - :py:exc:`KeyError` in strict mode is key is absent 
+                   of the :py:class:`Indexer` instance
+                   
+                 - :py:exc:`TypeError` if key is not an hashable value.
+        '''
         if dict.__contains__(self,key):
             return dict.__getitem__(self,key)
         elif strict:
@@ -47,25 +84,36 @@ class Indexer(dict):
             
     def __getitem__(self,key):
         '''
-        Return the integer key associated to a python value.
+        Implement the [] operateor to emulate the standard dictionnary
+        behaviour on :py:class:`Indexer` and returns the integer key 
+        associated to a python value.
         
-        @param key: the value to index
-        @type key: an hashable python value
-        @return: an unique integer value associated to the key
-        @rtype: int
+        Actually this method call the:py:meth:`getIndex` method in
+        non strict mode so it only raises an :py:exc:`TypeError` 
+        if key is not an hashable value.
+        
+        :param key: the value to index
+        :type key: an hashable python value
+        
+        :return: an unique integer value associated to the key
+        :rtype: int
+        
+        :raises: :py:exc:`TypeError` if **key** is not an hashable value.
         
         '''
         return self.getIndex(key)
         
     def __equal__(self,index):
         '''
-        Implement equa operator. Two Indexer instances are
-        equals if they are physically the same instance
-        @param index: the seconde Indexer
-        @type index: an Indexer instance
+        Implement equal  operator **==** for comparing two :py:class:`Indexer` instances. 
+        Two :py:class:`Indexer` instances are equals only if they are physically 
+        the same instance
         
-        @return: True is the two instances are the same
-        @rtype: bool
+        :param index: the second Indexer
+        :type index: an :py:class:`Indexer` instance
+        
+        :return: True is the two :py:class:`Indexer` instances are the same
+        :rtype: bool
         '''
         return id(self)==id(index)
         
@@ -74,23 +122,37 @@ class Graph(object):
     '''
     Class used to represent directed or undirected graph.
     
-    Only one edge can connect two nodes
+    .. warning::
+    
+        Only one edge can connect two nodes in a given direction.
+        
+    .. warning::
+        
+        Specifying nodes through their index seepud your code but as no check
+        is done on index value, it may result in inconsistency. So prefer the
+        use of node label to specify a node.
+    
+    
     '''
     def __init__(self,label='G',directed=False,indexer=None,nodes=None,edges=None):
         '''
-        Graph constructor.
-
-        @param label: Graph name, set to 'G' by default
-        @type label: str
-        @param directed: true for directed graph, set to False by defalt
-        @type directed: boolean
-        @param indexer: node label indexer
-        @type indexer: Indexer instance
-        @param nodes: set of nodes to add to the graph
-        @type nodes: iterable value
-        @param edges: set of edges to add to the graph
-        @type edges: iterable value
+        :param label: Graph name, set to 'G' by default
+        :type label: str
+        
+        :param directed: true for directed graph, set to False by defalt
+        :type directed: boolean
+        
+        :param indexer: node label indexer. This allows to define several graphs
+                        sharing the same indexer (see : :py:meth:`newEmpty`)
+        :type indexer: :py:class:`Indexer` 
+        
+        :param nodes: set of nodes to add to the graph
+        :type nodes: iterable value
+        
+        :param edges: set of edges to add to the graph
+        :type edges: iterable value
         '''
+        
         self._directed=directed
         if indexer is None:
             indexer = Indexer()
@@ -101,29 +163,90 @@ class Graph(object):
         self._label=label
         
     def newEmpty(self):
+        """
+        Build a new empty graph using the same :py:class:`Indexer` instance.
+        This allows two graph for sharing their vertices through their indices.
+        """
         n = Graph(self._label+"_compact",self._directed,self._index)
+        
         return n
         
     def addNode(self,node=None,index=None,**data):
+        '''
+        Add a new node or update an existing one.
+        
+        :param node: the new node label or the label of an existing node
+                     for updating it.
+        :type node:  an hashable python value
+        
+        :param index: the index of an existing node for updating it.
+        :type index: int
+        
+        :return: the index of the node
+        :rtype: int
+        
+        :raises: :py:exc:`IndexError` is index is not **None** and 
+                 corresponds to a not used index in this graph.
+        '''
         if index is None:
             index = self._index[node]
 
-        if index not in self._node:
-            self._node[index]=set()
+            if index not in self._node:
+                self._node[index]=set()
+        else:
+            if index not in self._node:
+                raise IndexError,"This index is not used in this graph"
 
         if data:
             if index in self._node_attrs:
                 self._node_attrs[index].update(data)
             else:
                 self._node_attrs[index]=dict(data)
+                
         return index
-            
+    
+    def __contains__(self,node):
+        try:
+            index = self._index.getIndex(node,strict=True)
+            r = index in self._node
+        except KeyError:
+            r=False
+        return r
+    
     def getNode(self,node=None,index=None):
+        """
+        :param node: a node label.
+        :type node:  an hashable python value
+        
+        :param index: the index of an existing node.
+        :type index: int
+        
+        .. note:: Index value are prevalent over node label.
+        
+        :return: the looked for node
+        :rtype: :py:class:`Node`
+        
+        :raises: :py:exc:`IndexError` if specified node lablel
+                 corresponds to a non-existing node.
+                 
+        .. warning:: no check on index value
+        """
         if index is None:
             index = self._index.getIndex(node, True)
         return Node(index,self)
     
     def getBestNode(self,estimator):
+        '''
+        Select the node maximizing the estimator function
+
+        :param estimator: the function to maximize
+        :type estimator: a function returning a numerical value and accepting one
+                         argument of type :py:class:`Node`
+                         
+        :return: the best node
+        :rtype: py:class:`Node`
+        '''
+
         bestScore=0
         best=None
         for n in self:
@@ -135,8 +258,25 @@ class Graph(object):
             
     
     def delNode(self,node=None,index=None):
+        """
+        Delete a node from a graph and all associated edges.
+        
+        :param node: a node label.
+        :type node:  an hashable python value
+        
+        :param index: the index of an existing node.
+        :type index: int
+        
+        .. note:: Index value are prevalent over node label.
+        
+        :raises: :py:exc:`IndexError` if specified node lablel
+                 corresponds to a non-existing node.
+                 
+        .. warning:: no check on index value
+        """
         if index is None:
             index = self._index[node]
+            
         for n in self._node:
             if n!=index:
                 e = self._node[n]
@@ -144,16 +284,41 @@ class Graph(object):
                     if (n,index) in self._edge_attrs:
                         del self._edge_attrs[(n,index)]
                     e.remove(index)
+                    
         e = self._node[index]
+        
         for n in e:
             if (index,n) in self._edge_attrs:
                 del self._edge_attrs[(index,n)]
+                
         del self._node[index]
         if index in self._node_attrs:
             del self._node_attrs[index]
                 
                     
     def addEdge(self,node1=None,node2=None,index1=None,index2=None,**data):
+        '''
+        Create a new edge in the graph between both the specified nodes.
+        
+        .. note:: Nodes can be specified using their label or their index in the graph
+        if both values are indicated the index is used.
+        
+        :param node1: The first vertex label
+        :type node1:  an hashable python value
+        :param node2: The second vertex label
+        :type node2:  an hashable python value
+        :param index1: The first vertex index
+        :type index1:  int
+        :param index2: The second vertex index
+        :type index2:  int
+
+        :raises: :py:exc:`IndexError` if one of both the specified node lablel
+                 corresponds to a non-existing node.
+                 
+
+        .. warning:: no check on index value
+        '''
+        
         index1=self.addNode(node1, index1)
         index2=self.addNode(node2, index2)
         
@@ -169,27 +334,59 @@ class Graph(object):
                 if not self._directed:
                     self._edge_attrs[(index2,index1)]=data
             else:
-                self._edge_attrs.update(data)
+                self._edge_attrs[(index2,index1)].update(data)
                 
         return (index1,index2)
     
     def getEdge(self,node1=None,node2=None,index1=None,index2=None):
         '''
+        Extract the :py:class:`Edge` instance linking two nodes of the graph. 
         
-        @param node1:
-        @type node1:
-        @param node2:
-        @type node2:
-        @param index1:
-        @type index1:
-        @param index2:
-        @type index2:
+        .. note:: Nodes can be specified using their label or their index in the graph
+        if both values are indicated the index is used.
+        
+        :param node1: The first vertex label
+        :type node1:  an hashable python value
+        :param node2: The second vertex label
+        :type node2:  an hashable python value
+        :param index1: The first vertex index
+        :type index1:  int
+        :param index2: The second vertex index
+        :type index2:  int
+
+        :raises: :py:exc:`IndexError` if one of both the specified node lablel
+                 corresponds to a non-existing node.
+                 
+
+        .. warning:: no check on index value
         '''
         node1=self.getNode(node1, index1)
         node2=self.getNode(node2, index2)
         return Edge(node1,node2)
     
     def delEdge(self,node1=None,node2=None,index1=None,index2=None):
+        """
+        Delete the edge linking node 1 to node 2.
+        
+        .. note:: Nodes can be specified using their label or their index in the graph
+        if both values are indicated the index is used.
+        
+        
+        :param node1: The first vertex label
+        :type node1:  an hashable python value
+        :param node2: The second vertex label
+        :type node2:  an hashable python value
+        :param index1: The first vertex index
+        :type index1:  int
+        :param index2: The second vertex index
+        :type index2:  int
+
+        :raises: :py:exc:`IndexError` if one of both the specified node lablel
+                 corresponds to a non-existing node.
+                 
+
+        .. warning:: no check on index value
+        """
         if index1 is None:
             index1 = self._index[node1]
         if index2 is None:
@@ -204,6 +401,21 @@ class Graph(object):
                     del self._node_attrs[(index2,index1)]
                     
     def edgeIterator(self,predicate=None):
+        """
+        Iterate through a set of selected vertices.
+        
+        :param predicate: a function allowing node selection. Default value
+                          is **None** and indicate that all nodes are selected.
+        :type predicate:  a function returning a boolean value
+                          and accepting one argument of class :py:class:`Edge`
+                          
+        :return: an iterator over selected edge 
+        :rtype: interator over :py:class:`Edge` instances
+        
+        .. seealso::
+            function :py:func:`selectEdgeAttributeFactory` for simple predicate.
+            
+        """
         for n1 in self._node:
             for n2 in self._node[n1]:
                 if self._directed or n1 <= n2:
@@ -213,12 +425,36 @@ class Graph(object):
                     
         
     def nodeIterator(self,predicate=None):
+        """
+        Iterate through a set of selected vertices.
+        
+        :param predicate: a function allowing edge selection. Default value
+                          is **None** and indicate that all edges are selected.
+        :type predicate:  a function returning a boolean value
+                          and accepting one argument of class :py:class:`Node`
+                          
+        :return: an iterator over selected nodes. 
+        :rtype: interator over :py:class:`Node` instances
+        
+        """
         for n in self._node:
             node = self.getNode(index=n)
             if predicate is None or predicate(node):
                 yield node
             
     def nodeIndexIterator(self,predicate=None):
+        """
+        Iterate through the indexes of a set of selected vertices.
+        
+        :param predicate: a function allowing edge selection. Default value
+                          is **None** and indicate that all edges are selected.
+        :type predicate:  a function returning a boolean value
+                          and accepting one argument of class :py:class:`Node`
+                          
+        :return: an iterator over selected node indices. 
+        :rtype: interator over `int`
+        
+        """
         for n in self._node:
             node = self.getNode(index=n)
             if predicate is None or predicate(node):
@@ -242,10 +478,10 @@ class Graph(object):
         for n in nodes:
             sub._node[n]=nodes & self._node[n]
             if n in self._node_attrs:
-                sub._node_attrs[n]=dict(self._node_attrs)
+                sub._node_attrs[n]=dict(self._node_attrs[n])
             for n2 in sub._node[n]:
                 if not self._directed:
-                    if n < n2:
+                    if n <= n2:
                         if (n,n2) in self._edge_attrs:
                             data=dict(self._edge_attrs[(n,n2)])
                             sub._edge_attrs[(n,n2)]=data
@@ -280,16 +516,20 @@ class Graph(object):
         return "%s %s {\n    %s\n\n    %s\n}" % (kw,self._label,nodes,edges)
         
 class Node(object):
+    """
+    Class used for representing one node or vertex in a graph
+    
+    """
     def __init__(self,index,graph):
-        '''
-        Constructor on Node instance. 
+        '''        
+        .. warning::
+            
+            :py:class:`Node` constructor is usualy called through the :py:class:`Graph` methods
         
-        Normaly this constructor is only used by Graph methods.
-        
-        @param index: Index of the node in the graph
-        @type index:  int
-        @param graph: graph instance owning the node
-        @type graph:  obitools.graph.Graph
+        :param index: Index of the node in the graph
+        :type index:  int
+        :param graph: graph instance owning the node
+        :type graph:  :py:class:`obitools.graph.Graph`
         '''
         self.index = index
         self.__graph = graph
@@ -298,7 +538,7 @@ class Node(object):
         '''
         return graph owning this node.
         
-        @rtype: L{obitools.graph.Graph}
+        :rtype: :py:class:`obitools.graph.Graph`
         '''
         return self.__graph
 
@@ -314,11 +554,11 @@ class Node(object):
         '''
         test is the node instance has a property named 'key'.
         
-        @param key: the name of a property
-        @type key: str
+        :param key: the name of a property
+        :type key: str
         
-        @return: True if the nade has a property named <key>
-        @rtype: bool
+        :return: True if the nade has a property named <key>
+        :rtype: bool
         '''
         if self.index in self.__graph._node_attrs:
             return key in self.__graph._node_attrs[self.index]
@@ -330,18 +570,18 @@ class Node(object):
         iterate through the nodes directly connected to
         this node.
         
-        @param nodePredicat: a function accepting one node as parameter
-                         and returning True if this node must be
+        :param nodePredicat: a function accepting one node as parameter
+                         and returning **True** if this node must be
                          returned by the iterator.
-        @type nodePredicat: function
+        :type nodePredicat: function
         
-        @param edgePredicat: a function accepting one edge as parameter
+        :param edgePredicat: a function accepting one edge as parameter
                          and returning True if the edge linking self and
                          the current must be considered.
-        @type edgePredicat: function
+        :type edgePredicat: function
         
         
-        @rtype: iterator on Node instances
+        :rtype: iterator on Node instances
         '''
         for n in self.neighbourIndexIterator(nodePredicat, edgePredicat):
             node = self.graph.getNode(index=n)
@@ -352,8 +592,10 @@ class Node(object):
         Return a set of node indexes directely connected
         to this node.
         
-        @attention: do not change this set unless you know
-                    exactly what you do.
+        .. warning:: 
+        
+                do not change this set unless you know
+                exactly what you do.
                     
         @rtype: set of int
         '''
@@ -364,17 +606,17 @@ class Node(object):
         iterate through the node indexes directly connected to
         this node.
         
-        @param nodePredicat: a function accepting one node as parameter
+        :param nodePredicat: a function accepting one node as parameter
                          and returning True if this node must be
                          returned by the iterator.
-        @type nodePredicat: function
+        :type nodePredicat: function
         
-        @param edgePredicat: a function accepting one edge as parameter
+        :param edgePredicat: a function accepting one edge as parameter
                          and returning True if the edge linking self and
                          the current must be considered.
-        @type edgePredicat: function
+        :type edgePredicat: function
         
-        @rtype: iterator on int
+        :rtype: iterator on int
         '''
         for n in self.neighbourIndexSet():
             if nodePredicat is None or nodePredicat(self.__graph.getNode(index=n)):
@@ -386,13 +628,13 @@ class Node(object):
         return count of edges linking this node to the
         set of nodes describes by their index in  nodeIndexes
         
-        @param nodeIndexes: set of node indexes. 
+        :param nodeIndexes: set of node indexes. 
                             if set to None, all nodes of the
                             graph are take into account.
                             Set to None by default.
-        @type nodeIndexes:  set of int
+        :type nodeIndexes:  set of int
         
-        @rtype: int
+        :rtype: int
         '''
         if nodeIndexes is None:
             return len(self.__graph._node[self.index])
@@ -403,18 +645,18 @@ class Node(object):
         '''
         Return the set of node index in the same connected component.
         
-        @param nodePredicat: a function accepting one node as parameter
+        :param nodePredicat: a function accepting one node as parameter
                          and returning True if this node must be
                          returned by the iterator.
-        @type nodePredicat: function
+        :type nodePredicat: function
         
-        @param edgePredicat: a function accepting one edge as parameter
+        :param edgePredicat: a function accepting one edge as parameter
                          and returning True if the edge linking self and
                          the current must be considered.
-        @type edgePredicat: function
+        :type edgePredicat: function
         
 
-        @rtype: set of int
+        :rtype: set of int
         '''
         cc=set([self.index])
         added = set(x for x in self.neighbourIndexIterator(nodePredicat, edgePredicat))
@@ -432,7 +674,7 @@ class Node(object):
         Iterate through the nodes in the same connected
         component.
         
-        @rtype: iterator on L{Node} instance
+        :rtype: iterator on :py:class:`Node` instance
         '''
         for c in self.componentIndexSet(nodePredicat, edgePredicat):
             yield self.graph.getNode(c)
@@ -444,11 +686,11 @@ class Node(object):
         only path linkink this node to one node listed in
         nodes
         
-        @param nodes: set of node index
-        @type nodes: iterable on int
+        :param nodes: set of node index
+        :type nodes: iterable on int
         
-        @return: an iterator on list of int describing path
-        @rtype: iterator on list of int
+        :return: an iterator on list of int describing path
+        :rtype: iterator on list of int
         '''
         if nodes is not None:
             nodes = set(nodes)
@@ -480,14 +722,14 @@ class Node(object):
         return one of the shortest path linking this
         node to specified node.
         
-        @param node: a node label or None
-        @param index: a node index or None. the parameter index  
+        :param node: a node label or None
+        :param index: a node index or None. the parameter index  
                      has a priority on the parameter node.
-        @type index: int
+        :type index: int
         
-        @return: list of node index corresponding to the path or None
+        :return: list of node index corresponding to the path or None
                  if no path exists.
-        @rtype: list of int or None 
+        :rtype: list of int or None 
         '''
         if index is None:
             index=self.graph.getNode(node).index
@@ -499,8 +741,8 @@ class Node(object):
         '''
         return the value of the <key> property of this node
         
-        @param key: the name of a property
-        @type key: str
+        :param key: the name of a property
+        :type key: str
         '''
         return self.__graph._node_attrs.get(self.index,{})[key]
     
@@ -509,11 +751,13 @@ class Node(object):
         set the value of a node property. In the property doesn't
         already exist a new property is added to this node.
         
-        @param key: the name of a property
-        @type key: str
-        @param value: the value of the property
+        :param key: the name of a property
+        :type key: str
+        :param value: the value of the property
         
-        @see: L{Node.__getitem__}
+        .. seealso:: 
+        
+            :py:meth:`Node.__getitem__`
         '''
         if self.index in self.__graph._node_attrs:
             data = self.__graph._node_attrs[self.index]
@@ -525,9 +769,11 @@ class Node(object):
         '''
         Count neighbour of this node
         
-        @rtype: int
+        :rtype: int
         
-        @see:  L{Node.degree}
+        .. seealso::  
+        
+            :py:meth:`Node.degree`
         '''
         return len(self.__graph._node[self.index])
     
@@ -535,9 +781,11 @@ class Node(object):
         '''
         iterate through neighbour of this node
         
-        @rtype: iterator in L{Node} instances
+        :rtype: iterator in :py:class:`Node` instances
         
-        @see: L{Node.neighbourIterator}
+        .. seealso::
+         
+            :py:meth:`Node.neighbourIterator`
         '''
         return self.neighbourIterator()
         
@@ -553,8 +801,17 @@ class Node(object):
         else:
             keys=''
             
-        return '%d  [label="%s" %s]' % (self.index,str(self.label).replace('"','\\"').replace('\n','\\n'),keys)       
-
+        return '%d  [label="%s" %s]' % (self.index,
+                                        str(self.label).replace('"','\\"').replace('\n','\\n'),
+                                        keys)      
+        
+    def keys(self):
+        if self.index in self.__graph._node_attrs:
+            k = self.__graph._node_attrs[self.index].keys()
+        else:
+            k=[]
+        return k
+    
     label = property(getLabel, None, None, "Label of the node")
 
     graph = property(getGraph, None, None, "Graph owning this node")
@@ -562,22 +819,40 @@ class Node(object):
         
     
 class Edge(object):
+    """
+    Class used for representing one edge of a graph
+    
+    """
+
     def __init__(self,node1,node2):
+        '''
+        .. warning::
+            
+            :py:class:`Edge` constructor is usualy called through the :py:class:`Graph` methods
+            
+        :param node1: First node likend by the edge
+        :type node1:  :py:class:`Node`
+        :param node2: Seconde node likend by the edge
+        :type node2:  :py:class:`Node`
+        '''
         self.node1 = node1
         self.node2 = node2
 
     def getGraph(self):
+        """
+        Return the :py:class:`Graph` instance owning this edge.
+        """
         return self.node1.graph
 
     def has_key(self,key):
         '''
-        test is the edge instance has a property named 'key'.
+        test is the :py:class:`Edge` instance has a property named **key**.
         
-        @param key: the name of a property
-        @type key: str
+        :param key: the name of a property
+        :type key: str
         
-        @return: True if the edge has a property named <key>
-        @rtype: bool
+        :return: True if the edge has a property named <key>
+        :rtype: bool
         '''
         if (self.node1.index,self.node2.index) in self.graph._edge_attrs:
             return key in self.graph._edge_attrs[(self.node1.index,self.node2.index)]
@@ -625,35 +900,49 @@ class Edge(object):
     
         
 class DiGraph(Graph):
+    """
+    :py:class:`DiGraph class`is a specialisation of the :py:class:`Graph` class
+    dedicated to directed graph representation
+    
+    .. seealso::
+        
+        :py:class:`UndirectedGraph`
+        
+    """
     def __init__(self,label='G',indexer=None,nodes=None,edges=None):
         '''
-        Directed Graph constructor.
-
-        @param label: Graph name, set to 'G' by default
-        @type label: str
-        @param indexer: node label indexer
-        @type indexer: Indexer instance
-        @param nodes: set of nodes to add to the graph
-        @type nodes: iterable value
-        @param edges: set of edges to add to the graph
-        @type edges: iterable value
+        :param label: Graph name, set to 'G' by default
+        :type label: str
+        :param indexer: node label indexer
+        :type indexer: Indexer instance
+        :param nodes: set of nodes to add to the graph
+        :type nodes: iterable value
+        :param edges: set of edges to add to the graph
+        :type edges: iterable value
         '''
         
         Graph.__init__(self, label, True, indexer, nodes, edges)
         
 class UndirectedGraph(Graph):
+    """
+    :py:class:`UndirectGraph class`is a specialisation of the :py:class:`Graph` class
+    dedicated to undirected graph representation
+    
+    .. seealso::
+        
+        :py:class:`DiGraph`
+        
+    """
     def __init__(self,label='G',indexer=None,nodes=None,edges=None):
         '''
-        Directed Graph constructor.
-
-        @param label: Graph name, set to 'G' by default
-        @type label: str
-        @param indexer: node label indexer
-        @type indexer: Indexer instance
-        @param nodes: set of nodes to add to the graph
-        @type nodes: iterable value
-        @param edges: set of edges to add to the graph
-        @type edges: iterable value
+        :param label: Graph name, set to 'G' by default
+        :type label: str
+        :param indexer: node label indexer
+        :type indexer: Indexer instance
+        :param nodes: set of nodes to add to the graph
+        :type nodes: iterable value
+        :param edges: set of edges to add to the graph
+        :type edges: iterable value
         '''
         
         Graph.__init__(self, label, False, indexer, nodes, edges)
@@ -661,6 +950,13 @@ class UndirectedGraph(Graph):
    
         
 def selectEdgeAttributeFactory(attribut,value):
+    """
+    This function help in building predicat function usable for selecting edge
+    in the folowing :py:class:`Graph` methods :
+    
+        - :py:meth:`Graph.edgeIterator`
+        
+    """
     def selectEdge(e):
         return attribut in e and e[attribut]==value
     return selectEdge   
