@@ -2,6 +2,12 @@ import sys
 from obitools.options.taxonomyfilter import loadTaxonomyDatabase
 def addSequenceEditTagOptions(optionManager):
     
+    optionManager.add_option('--rank',
+                             action="store_true", dest='addrank',
+                             default=False,
+                             help="add a rank attribute to the sequence "
+                                  "indicating the sequence position in the input data")
+
     optionManager.add_option('-R','--rename-tag',
                              action="append", 
                              dest='renameTags',
@@ -116,6 +122,7 @@ def sequenceTaggerGenerator(options):
     clear    = options.clear
     keep     = set(options.keep)
     length   = options.length
+    counter  = [0]
     loadTaxonomyDatabase(options)
     if options.taxonomy is not None:
         annoteRank=options.taxonrank
@@ -128,6 +135,10 @@ def sequenceTaggerGenerator(options):
         parts = False
     
     def sequenceTagger(seq):
+        
+        if counter[0]>=0:
+            counter[0]+=1
+        
         if clear or keep:
             ks = seq.keys()
             for k in ks:
@@ -141,47 +152,7 @@ def sequenceTaggerGenerator(options):
                 if o in seq:
                     seq[n]=seq[o]
                     del seq[o]
-        for i,v in toSet:
-            try:
-                val = eval(v,{'sequence':seq},seq)
-            except Exception,e:
-                if options.onlyValid:
-                    raise e
-                val = v
-            seq[i]=val
-            
-        if length:
-            seq['seqLength']=len(seq)
-            
-        if newId is not None:
-            try:
-                val = eval(newId,{'sequence':seq},seq)
-            except Exception,e:
-                if options.onlyValid:
-                    raise e
-                val = newId
-            seq.id=val
-        if newDef is not None:
-            try:
-                val = eval(newDef,{'sequence':seq},seq)
-            except Exception,e:
-                if options.onlyValid:
-                    raise e
-                val = newDef
-            seq.definition=val
-            
-        if newSeq is not None:
-            try:
-                val = eval(newSeq,{'sequence':seq},seq)
-            except Exception,e:
-                if options.onlyValid:
-                    raise e
-                val = newSeq
-            if hasattr(seq, '_seq'):
-                seq._seq=str(val).lower()
-                if 'seqLength' in seq:
-                    seq['seqLength']=len(seq)
-
+                    
         for rank in annoteRank:
             if 'taxid' in seq:
                 taxid = seq['taxid']
@@ -197,6 +168,70 @@ def sequenceTaggerGenerator(options):
         if parts and seq.id in parts:   
             seq['cluster']=parts[seq.id]
             
+        if options.addrank:
+            seq['rank']=counter[0]
+
+        for i,v in toSet:
+            try:
+                if options.taxonomy is not None:
+                    environ = {'taxonomy' : options.taxonomy,'sequence':seq, 'counter':counter[0]}
+                else:
+                    environ = {'sequence':seq, 'counter':counter[0]}
+                
+                val = eval(v,environ,seq)
+            except Exception,e:
+                if options.onlyValid:
+                    raise e
+                val = v
+            seq[i]=val
+            
+        if length:
+            seq['seqLength']=len(seq)
+            
+        if newId is not None:
+            try:
+                if options.taxonomy is not None:
+                    environ = {'taxonomy' : options.taxonomy,'sequence':seq, 'counter':counter[0]}
+                else:
+                    environ = {'sequence':seq, 'counter':counter[0]}
+                
+                val = eval(newId,environ,seq)
+            except Exception,e:
+                if options.onlyValid:
+                    raise e
+                val = newId
+            seq.id=val
+        if newDef is not None:
+            try:
+                if options.taxonomy is not None:
+                    environ = {'taxonomy' : options.taxonomy,'sequence':seq, 'counter':counter[0]}
+                else:
+                    environ = {'sequence':seq, 'counter':counter[0]}
+                
+                val = eval(newDef,environ,seq)
+            except Exception,e:
+                if options.onlyValid:
+                    raise e
+                val = newDef
+            seq.definition=val
+            
+        if newSeq is not None:
+            try:
+                if options.taxonomy is not None:
+                    environ = {'taxonomy' : options.taxonomy,'sequence':seq, 'counter':counter[0]}
+                else:
+                    environ = {'sequence':seq, 'counter':counter[0]}
+                
+                val = eval(newSeq,environ,seq)
+            except Exception,e:
+                if options.onlyValid:
+                    raise e
+                val = newSeq
+            if hasattr(seq, '_seq'):
+                seq._seq=str(val).lower()
+                if 'seqLength' in seq:
+                    seq['seqLength']=len(seq)
+            
         return seq
     
     return sequenceTagger
\ No newline at end of file
