@@ -88,30 +88,46 @@ def fileSize(file):
     file.seek(pos,0)
     return length
 
-def progressBar(pos,max,reset=False,head='',delta=[]):
+def progressBar(pos,maxi,reset=False,head='',delta=[],step=[1,0,0]):
     if reset:
         del delta[:]
     if not delta:
         delta.append(time.time())
         delta.append(time.time())
-    assert max>0
+    assert maxi>0
 
-    delta[1]=time.time()
-    elapsed = delta[1]-delta[0]
-    percent = float(pos)/max * 100
-    remain = time.gmtime(elapsed / percent * (100-percent))
-    days = remain.tm_yday - 1
-    hour = remain.tm_hour
-    min  = remain.tm_min
-    sec  = remain.tm_sec
-    if days:
-        remain = "%d days %02d:%02d:%02d" % (days,hour,min,sec)
+    step[1]+=1
+    if step[1] % step[0] == 0:
+        step[1]=1
+        newtime = time.time()
+        d = newtime-delta[1]
+        if d < 0.2:
+            step[0]*=2
+        elif d > 0.4 and step[0]>1:
+            step[0]/=2
+            
+        delta[1]=newtime
+        elapsed = delta[1]-delta[0]
+        
+        if callable(pos):
+            pos=pos()
+        percent = float(pos)/maxi * 100
+        remain = time.gmtime(elapsed / percent * (100-percent))
+        days = remain.tm_yday - 1
+        hour = remain.tm_hour
+        minu  = remain.tm_min
+        sec  = remain.tm_sec
+        if days:
+            remain = "%d days %02d:%02d:%02d" % (days,hour,minu,sec)
+        else:
+            remain = "%02d:%02d:%02d" % (hour,minu,sec)        
+        bar = '#' * int(percent/2)
+        step[2]=(step[2]+1) % 4
+        bar+= '|/-\\'[step[2]]
+        bar+= ' ' * (50 - int(percent/2))
+        sys.stderr.write('\r%s %5.1f %% |%s] remain : %s' %(head,percent,bar,remain))
     else:
-        remain = "%02d:%02d:%02d" % (hour,min,sec)        
-    bar = '#' * int(percent/2)
-    bar+= '|/-\\-'[pos % 5]
-    bar+= ' ' * (50 - int(percent/2))
-    sys.stderr.write('\r%s %5.1f %% |%s] remain : %s' %(head,percent,bar,remain))
+        step[1]+=1
 
 def endLessIterator(endedlist):
     for x in endedlist:
@@ -169,7 +185,10 @@ def skipWhiteLineIterator(lineiterator):
 
 class ColumnFile(object):
     
-    def __init__(self,stream,sep=None,strip=True,types=None,skip=None,head=None,extra=None,extraformat='([a-zA-Z]\w*) *= *([^;]+);'):
+    def __init__(self,stream,sep=None,strip=True,
+                 types=None,skip=None,head=None,
+                 extra=None,
+                 extraformat='([a-zA-Z]\w*) *= *([^;]+);'):
         self._stream = universalOpen(stream)
         self._delimiter=sep
         self._strip=strip
@@ -201,6 +220,13 @@ class ColumnFile(object):
         return self
     
     def next(self):
+        
+        def cast(txt,type):
+            try:
+                v = type(txt)
+            except:
+                v=None
+            return v
         ligne = self._stream.next()
         if self._skip is not None:
             while ligne[0:self._lskip]==self._skip:
@@ -218,7 +244,7 @@ class ColumnFile(object):
             data = [x.strip() for x in data]
         if self._types:
             it = endLessIterator(self._types)
-            data = [x[1](x[0]) for x in ((y,it.next()) for y in data)]
+            data = [cast(*x) for x in ((y,it.next()) for y in data)]
         if self._head is not None:
             data=dict(map(None, self._head,data))
             if extra is not None:
